package com.my.system.service;

import java.util.List;
import com.my.common.core.domain.TreeSelect;
import com.my.common.core.domain.entity.SysDept;

/**
 * 部门管理 服务层
 * 
 * @author ruoyi
 */
public interface ISysDeptService
{
    /**
     * 查询部门管理数据
     * 
     * @param dept 部门信息
     * @return 部门信息集合
     */
    public List<SysDept> selectDeptList(SysDept dept);

    /**
     * 查询部门树结构信息
     * 
     * @param dept 部门信息
     * @return 部门树信息集合
     */
    public List<TreeSelect> selectDeptTreeList(SysDept dept);

    /**
     * 构建前端所需要树结构
     * 
     * @param depts 部门列表
     * @return 树结构列表
     */
    public List<SysDept> buildDeptTree(List<SysDept> depts);

    /**
     * 构建前端所需要下拉树结构
     * 
     * @param depts 部门列表
     * @return 下拉树结构列表
     */
    public List<TreeSelect> buildDeptTreeSelect(List<SysDept> depts);

    /**
     * 根据角色ID查询部门树信息
     * 
     * @param roleId 角色ID
     * @return 选中部门列表
     */
    public List<Long> selectDeptListByRoleId(Long roleId);

    /**
     * 根据部门ID查询信息
     * 
     * @param deptId 部门ID
     * @return 部门信息
     */
    public SysDept selectDeptById(Long deptId);

    /**
     * 根据ID查询所有子部门（正常状态）
     * 
     * @param deptId 部门ID
     * @return 子部门数
     */
    public int selectNormalChildrenDeptById(Long deptId);

    /**
     * 是否存在部门子节点
     * 
     * @param deptId 部门ID
     * @return 结果
     */
    public boolean hasChildByDeptId(Long deptId);

    /**
     * 查询部门是否存在用户
     * 
     * @param deptId 部门ID
     * @return 结果 true 存在 false 不存在
     */
    public boolean checkDeptExistUser(Long deptId);

    /**
     * 校验部门名称是否唯一
     * 
     * @param dept 部门信息
     * @return 结果
     */
    public boolean checkDeptNameUnique(SysDept dept);

    /**
     * 校验部门是否有数据权限
     * 
     * @param deptId 部门id
     */
    public void checkDeptDataScope(Long deptId);

     /**
     * 新增保存部门信息
     * 
     * @param dept 部门信息
     * @return 结果
     */
    public int insertDept(SysDept dept);

    /**
     * 修改保存部门信息
     * 
     * @param dept 部门信息
     * @return 结果
     */
    public int updateDept(SysDept dept);

    /**
     * 删除部门管理信息
     * 
     * @param deptId 部门ID
     * @return 结果
     */
    public int deleteDeptById(Long deptId);

    /**
     * 机构同步前先，先把所有机构的删除状态设置为删除
     */
    public void updateDelFlag();

    /**
     * 机构同步时，先获取所有机构
     * @return
     */
    public List<SysDept> selectAllDept();

    /**
     * 机构同步完成后处理父机构 Long id 及祖级列表
     */
    public void updateAllDeptId();

    /**
     * 更新祖级列表
     * @param sortLen
     */
    public int updateAllAncestors(int sortLen);

    /**
     * 根据部门ID查询信息
     *
     * @param code 部门code
     * @return 部门信息
     */
    public SysDept selectDeptByCode(String code);
    public SysDept selectDeptByName(String name);
    /**
     * 根据部门ID查询隶属关
     *
     * @param deptId 部门ID
     * @return 部门信息
     */
    public SysDept selectOrgDeptById(Long deptId);

    /**
     * 新增保存部门信息
     *
     * @param dept 部门信息
     * @return 结果
     */
    public int insertH4aDept(SysDept dept);

    /**
     * 修改保存部门信息
     *
     * @param dept 部门信息
     * @return 结果
     */
    public int updateH4aDept(SysDept dept);

    /**
     * 获取用户所属单位及下属部门
     * @param deptId
     * @return
     */
    public List<SysDept> userDept(Long deptId );
    public List<SysDept> userDeptOrg(Long deptId );

    /**
     * 获取总关、隶属海关、缉私局、事业单位四大类 机构
     * @return
     */
    public List<SysDept> selectOrg(Long orgId);


    /**
     * 获取用户所属单位
     * @param ancestors 用户所属部门的祖级列表
     * @return
     */
    public SysDept getUserOrg(String ancestors);

    /** change useFlag */
    public int changeUseFlag(Long deptId, String useFlag);

}
